--------------------------------------------------------------------------------
-- Takes the two's complement form of the time differences and outputs a
-- magnitude and a sign for display on the Digital I/O.
--
-- Ilya Belopolski
-- 19 January, 2012
--------------------------------------------------------------------------------

library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity WriteMagSign is port
    (
    CLK : in std_logic;

    BNC0_DIFF : in std_logic_vector (31 downto 0);
    BNC1_DIFF : in std_logic_vector (31 downto 0);
    BNC2_DIFF : in std_logic_vector (31 downto 0);
    BNC3_DIFF : in std_logic_vector (31 downto 0);
    BNC4_DIFF : in std_logic_vector (31 downto 0);
    BNC5_DIFF : in std_logic_vector (31 downto 0);
    BNC6_DIFF : in std_logic_vector (31 downto 0);

    BNC0_DIFF_MAG  : out std_logic_vector (31 downto 0);
    BNC0_DIFF_SIGN : out std_logic;
    BNC1_DIFF_MAG  : out std_logic_vector (31 downto 0);
    BNC1_DIFF_SIGN : out std_logic;
    BNC2_DIFF_MAG  : out std_logic_vector (31 downto 0);
    BNC2_DIFF_SIGN : out std_logic;
    BNC3_DIFF_MAG  : out std_logic_vector (31 downto 0);
    BNC3_DIFF_SIGN : out std_logic;
    BNC4_DIFF_MAG  : out std_logic_vector (31 downto 0);
    BNC4_DIFF_SIGN : out std_logic;
    BNC5_DIFF_MAG  : out std_logic_vector (31 downto 0);
    BNC5_DIFF_SIGN : out std_logic;
    BNC6_DIFF_MAG  : out std_logic_vector (31 downto 0);
    BNC6_DIFF_SIGN : out std_logic

    );
end entity WriteMagSign;

architecture ark of WriteMagSign is

     signal diffdiff0 : signed(31 downto 0);
     signal diffdiff1 : signed(31 downto 0);
     signal diffdiff2 : signed(31 downto 0);
     signal diffdiff3 : signed(31 downto 0);
     signal diffdiff4 : signed(31 downto 0);
     signal diffdiff5 : signed(31 downto 0);
     signal diffdiff6 : signed(31 downto 0);

     signal off0 : signed(31 downto 0);
     signal off1 : signed(31 downto 0);
     signal off2 : signed(31 downto 0);
     signal off3 : signed(31 downto 0);
     signal off4 : signed(31 downto 0);
     signal off5 : signed(31 downto 0);
     signal off6 : signed(31 downto 0);

     signal offdiff0 : std_logic_vector(31 downto 0);
     signal offdiff1 : std_logic_vector(31 downto 0);
     signal offdiff2 : std_logic_vector(31 downto 0);
     signal offdiff3 : std_logic_vector(31 downto 0);
     signal offdiff4 : std_logic_vector(31 downto 0);
     signal offdiff5 : std_logic_vector(31 downto 0);
     signal offdiff6 : std_logic_vector(31 downto 0);

begin

     diffdiff0 <= signed(BNC0_DIFF) when rising_edge(CLK);
     diffdiff1 <= signed(BNC1_DIFF) when rising_edge(CLK);
     diffdiff2 <= signed(BNC2_DIFF) when rising_edge(CLK);
     diffdiff3 <= signed(BNC3_DIFF) when rising_edge(CLK);
     diffdiff4 <= signed(BNC4_DIFF) when rising_edge(CLK);
     diffdiff5 <= signed(BNC5_DIFF) when rising_edge(CLK);
     diffdiff6 <= signed(BNC6_DIFF) when rising_edge(CLK);

     finddiff : process (CLK) is
     begin
          if rising_edge (CLK) then

               -- Input 0
               if (diffdiff0(31) = '1') then
                  off0 <= X"00000000" - diffdiff0;
               else
                  off0 <= diffdiff0;
               end if;

               -- Input 1
               if (diffdiff1(31) = '1') then
                  off1 <= X"00000000" - diffdiff1;
               else
                  off1 <= diffdiff1;
               end if;

               -- Input 2
               if (diffdiff2(31) = '1') then
                  off2 <= X"00000000" - diffdiff2;
               else
                  off2 <= diffdiff2;
               end if;

               -- Input 3
               if (diffdiff3(31) = '1') then
                  off3 <= X"00000000" - diffdiff3;
               else
                  off3 <= diffdiff3;
               end if;

               -- Input 4
               if (diffdiff4(31) = '1') then
                  off4 <= X"00000000" - diffdiff4;
               else
                  off4 <= diffdiff4;
               end if;

               -- Input 5
               if (diffdiff5(31) = '1') then
                  off5 <= X"00000000" - diffdiff5;
               else
                  off5 <= diffdiff5;
               end if;

               -- Input 6
               if (diffdiff6(31) = '1') then
                  off6 <= X"00000000" - diffdiff6;
               else
                  off6 <= diffdiff6;
               end if;

          end if;

     end process finddiff;

     offdiff0 <= std_logic_vector(off0);
     offdiff1 <= std_logic_vector(off1);
     offdiff2 <= std_logic_vector(off2);
     offdiff3 <= std_logic_vector(off3);
     offdiff4 <= std_logic_vector(off4);
     offdiff5 <= std_logic_vector(off5);
     offdiff6 <= std_logic_vector(off6);

     BNC0_DIFF_MAG <= offdiff0(5 downto 0) & offdiff0(31 downto 6);
     BNC1_DIFF_MAG <= offdiff1(5 downto 0) & offdiff1(31 downto 6);
     BNC2_DIFF_MAG <= offdiff2(5 downto 0) & offdiff2(31 downto 6);
     BNC3_DIFF_MAG <= offdiff3(5 downto 0) & offdiff3(31 downto 6);
     BNC4_DIFF_MAG <= offdiff4(5 downto 0) & offdiff4(31 downto 6);
     BNC5_DIFF_MAG <= offdiff5(5 downto 0) & offdiff5(31 downto 6);
     BNC6_DIFF_MAG <= offdiff6(5 downto 0) & offdiff6(31 downto 6);

     BNC0_DIFF_SIGN <= std_logic(diffdiff0(31));
     BNC1_DIFF_SIGN <= std_logic(diffdiff1(31));
     BNC2_DIFF_SIGN <= std_logic(diffdiff2(31));
     BNC3_DIFF_SIGN <= std_logic(diffdiff3(31));
     BNC4_DIFF_SIGN <= std_logic(diffdiff4(31));
     BNC5_DIFF_SIGN <= std_logic(diffdiff5(31));
     BNC6_DIFF_SIGN <= std_logic(diffdiff6(31));

end architecture ark;





















